Option Compare Binary
Option Explicit On
Option Strict On

Imports MvpsOrg.Dotnet.Win32.ResLib.NativeMethods
Imports System
Imports System.Runtime.InteropServices

''' <summary>Represents a Win32 resource identifier.</summary>
Public MustInherit Class ResourceId

    ''' <summary>Initializes a new instance of the class.</summary>
    Friend Sub New()
        '
    End Sub

    ''' <summary>
    '''   Creates a resource identifier of appropriate type based on the
    '''   specified value.
    ''' </summary>
    ''' <param name="Value">
    '''   The string pointer the resource identifier should be created
    '''   from.
    ''' </param>
    ''' <returns>
    '''   A resource identifier object of appropriate type whose value is
    '''   equal to the specified value.
    ''' </returns>
    ''' <remarks>
    '''   <paramref name="Value"/> must be either a valid integer resource
    '''   identifier or a valid string pointer pointing to the name of the
    '''   resource identifier.
    ''' </remarks>
    Public Shared Function FromLPStr(ByVal Value As IntPtr) As ResourceId
        If NativeMethods.IS_INTRESOURCE(Value) Then
            Return New IntegerResourceId(Value)
        Else
            Return New StringResourceId(Marshal.PtrToStringAuto(Value))
        End If
    End Function

    ''' <summary>
    '''   Determines whether the two specified objects are equal.
    ''' </summary>
    ''' <param name="lhs">
    '''   The value on the left hand side of the comparison.
    ''' </param>
    ''' <param name="rhs">
    '''   The value on the right hand side of the comparison.
    ''' </param>
    ''' <returns>
    '''   <c>True</c> if the two specified objects are equal; otherwise,
    '''   <c>False</c>.
    ''' </returns>
    Public Shared Operator =( _
        ByVal lhs As ResourceId, _
        ByVal rhs As ResourceId _
    ) As Boolean
        If lhs Is rhs Then
            Return True
        ElseIf lhs Is Nothing OrElse rhs Is Nothing Then
            Return False
        ElseIf lhs.GetType() IsNot rhs.GetType() Then
            Return False
        ElseIf TypeOf lhs Is IntegerResourceId Then
            Return _
                DirectCast(lhs, IntegerResourceId).Value = _
                DirectCast(rhs, IntegerResourceId).Value
        ElseIf TypeOf lhs Is StringResourceId Then
            Return _
                DirectCast(lhs, StringResourceId).Value = _
                DirectCast(rhs, StringResourceId).Value
        Else
            Throw New NotSupportedException(My.Resources.ExceptionOperationNotSupportedForTypesOfArguments)
        End If
    End Operator

    ''' <summary>
    '''   Determines whether the two specified objects are different.
    ''' </summary>
    ''' <param name="lhs">
    '''   The value on the left hand side of the comparison.
    ''' </param>
    ''' <param name="rhs">
    '''   The value on the right hand side of the comparison.
    ''' </param>
    ''' <returns>
    '''   <c>True</c> if the two specified objects are different;
    '''   otherwise, <c>False</c>.
    ''' </returns>
    Public Shared Operator <>( _
        ByVal lhs As ResourceId, _
        ByVal rhs As ResourceId _
    ) As Boolean
        Return Not lhs = rhs
    End Operator

    ''' <summary>
    '''   Determines whether the specified object is equal to the current
    '''   object.
    ''' </summary>
    ''' <param name="obj">
    '''   The object to compare with the current object.
    ''' </param>
    ''' <returns>
    '''   <c>True</c> if the specified object is equal to the current
    '''   object; otherwise, <c>False</c>.
    ''' </returns>
    Public Overrides Function Equals(ByVal obj As Object) As Boolean
        If obj Is Nothing Then
            Return False
        ElseIf obj.GetType() IsNot Me.GetType() Then
            Return False
        ElseIf TypeOf obj Is IntegerResourceId Then
            Return _
                DirectCast(obj, IntegerResourceId).Value = _
                DirectCast(Me, IntegerResourceId).Value
        ElseIf TypeOf obj Is StringResourceId Then
            Return _
                DirectCast(obj, StringResourceId).Value = _
                DirectCast(Me, StringResourceId).Value
        Else
            Throw New NotSupportedException()
        End If
    End Function

    ''' <summary>
    '''   Serves as a hash function for this type. The return value is
    '''   suitable for use in hashing algorithms and data structures like a
    '''   hash table.
    ''' </summary>
    ''' <returns>A hash code for the current object.</returns>
    Public Overrides Function GetHashCode() As Integer
        Return MyBase.GetHashCode()
    End Function

    ''' <summary>Create process manifest resource identifier.</summary>
    Public Shared ReadOnly CreateProcessManifest As New IntegerResourceId(RESOURCE_ID_Constants.CREATEPROCESS_MANIFEST_RESOURCE_ID)

    ''' <summary>Isolation-aware manifest resource identifier.</summary>
    Public Shared ReadOnly IsolationAwareManifest As New IntegerResourceId(RESOURCE_ID_Constants.ISOLATIONAWARE_MANIFEST_RESOURCE_ID)

    ''' <summary>
    '''   Isolation-aware no-static import manifest resource identifier.
    ''' </summary>
    Public Shared ReadOnly IsolationAwareNoStaticImportManifest As New IntegerResourceId(RESOURCE_ID_Constants.ISOLATIONAWARE_NOSTATICIMPORT_MANIFEST_RESOURCE_ID)

    ''' <summary>Minimum reserved manifest resource identifier.</summary>
    Public Shared ReadOnly MinimumReservedManifest As New IntegerResourceId(RESOURCE_ID_Constants.MINIMUM_RESERVED_MANIFEST_RESOURCE_ID)

    ''' <summary>Maximum reserved manifest resource identifier.</summary>
    Public Shared ReadOnly MaximumReservedManifest As New IntegerResourceId(RESOURCE_ID_Constants.MAXIMUM_RESERVED_MANIFEST_RESOURCE_ID)
End Class